package de.ozgcloud.apilib.file.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.errorhandling.NotFoundException;
import de.ozgcloud.apilib.file.OzgCloudFileTestFactory;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;

class GrpcOzgCloudFileServiceTest {

	@InjectMocks
	private GrpcOzgCloudFileService service;

	@Mock
	private BinaryFileServiceBlockingStub blockingStub;
	@Spy
	private OzgCloudFileMapper mapper = Mappers.getMapper(OzgCloudFileMapper.class);

	@Nested
	class TestGetFile {

		@Nested
		class CallGetFile {
			@BeforeEach
			void init() {
				when(blockingStub.withInterceptors(any())).thenReturn(blockingStub);
				when(blockingStub.findBinaryFilesMetaData(any())).thenReturn(GrpcFindFilesResponse.newBuilder().addFile(GrpcOzgFileTestFactory.create()).build());
			}

			@Test
			void shouldCallStub() {
				service.getFile(OzgCloudFileTestFactory.ID);

				verify(blockingStub).findBinaryFilesMetaData(any());
			}

			@Test
			void shouldCallMapper() {
				service.getFile(OzgCloudFileTestFactory.ID);

				verify(mapper).fromGrpc(any());
			}
		}

		@Nested
		class BuildFindFileRequest {

			@Test
			void shouldAddFileId() {
				var request = service.buildFindFileRequest(OzgCloudFileTestFactory.ID);

				assertThat(request.getFileIdList()).hasSize(1).first().isEqualTo(OzgCloudFileTestFactory.ID.toString());
			}
		}

		@Nested
		class GetSingleResponse {
			@Test
			void shouldReturnSingleFile() {
				var result = service.getSingleResponse(GrpcFindFilesResponseTestFactory.create(), OzgCloudFileTestFactory.ID);

				assertThat(result).usingRecursiveComparison().isEqualTo(GrpcOzgFileTestFactory.create());
			}

			@Test
			void shouldReturnFirstFile() {
				var response = GrpcFindFilesResponseTestFactory.createBuilder().addFile(GrpcOzgFileTestFactory.createBuilder().setId("2").build())
						.build();

				var result = service.getSingleResponse(response, OzgCloudFileTestFactory.ID);

				assertThat(result).usingRecursiveComparison().isEqualTo(GrpcOzgFileTestFactory.create());
			}

			@Test
			void shouldThrowNotFound() {
				var response = GrpcFindFilesResponseTestFactory.createBuilder().clearFile().build();

				assertThatThrownBy(() -> service.getSingleResponse(response, OzgCloudFileTestFactory.ID))
						.isInstanceOf(NotFoundException.class);
			}
		}
	}

}
